"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.config = config;
/**
 * Utility function to make it easy to strictly type your "Flat" config file
 * @example
 * ```js
 * // @ts-check
 *
 * import eslint from '@eslint/js';
 * import tseslint from 'typescript-eslint';
 *
 * export default tseslint.config(
 *   eslint.configs.recommended,
 *   tseslint.configs.recommended,
 *   {
 *     rules: {
 *       '@typescript-eslint/array-type': 'error',
 *     },
 *   },
 * );
 * ```
 */
function config(...configs) {
    const flattened = 
    // @ts-expect-error -- intentionally an infinite type
    configs.flat(Infinity);
    return flattened.flatMap((configWithExtends, configIndex) => {
        const { extends: extendsArr, ...config } = configWithExtends;
        if (extendsArr == null || extendsArr.length === 0) {
            return config;
        }
        const extendsArrFlattened = extendsArr.flat(Infinity);
        const undefinedExtensions = extendsArrFlattened.reduce((acc, extension, extensionIndex) => {
            const maybeExtension = extension;
            if (maybeExtension == null) {
                acc.push(extensionIndex);
            }
            return acc;
        }, []);
        if (undefinedExtensions.length) {
            const configName = configWithExtends.name != null
                ? `, named "${configWithExtends.name}",`
                : ' (anonymous)';
            const extensionIndices = undefinedExtensions.join(', ');
            throw new Error(`Your config at index ${configIndex}${configName} contains undefined` +
                ` extensions at the following indices: ${extensionIndices}.`);
        }
        const configArray = [];
        for (const extension of extendsArrFlattened) {
            const name = [config.name, extension.name].filter(Boolean).join('__');
            if (isPossiblyGlobalIgnores(extension)) {
                // If it's a global ignores, then just pass it along
                configArray.push({
                    ...extension,
                    ...(name && { name }),
                });
            }
            else {
                configArray.push({
                    ...extension,
                    ...(config.files && { files: config.files }),
                    ...(config.ignores && { ignores: config.ignores }),
                    ...(name && { name }),
                });
            }
        }
        // If the base config could form a global ignores object, then we mustn't include
        // it in the output. Otherwise, we must add it in order for it to have effect.
        if (!isPossiblyGlobalIgnores(config)) {
            configArray.push(config);
        }
        return configArray;
    });
}
/**
 * This utility function returns false if the config objects contains any field
 * that would prevent it from being considered a global ignores object and true
 * otherwise. Note in particular that the `ignores` field may not be present and
 * the return value can still be true.
 */
function isPossiblyGlobalIgnores(config) {
    return Object.keys(config).every(key => ['name', 'ignores'].includes(key));
}
